/* 
 * Functions.h
 * 
 * Copyright (c) 2003 Enrico Sersale <enrico@imago.ro>
 *
 * Author: Enrico Sersale <enrico@imago.ro>
 * Date: August 2003
 *
 * This file is part of the GNUstep Position application
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

 
#ifndef FUNCTIONS_H
#define FUNCTIONS_H

#include <projects.h>

#ifndef METRIC
// #define PI 3.14159265358979323846264338327950288
// #define PI 3.14159265358979323846
// #define PI 3.1415926535898

  #define degrToRad(x) (x * PI / 180)
  #define sindegr(x) sin(degrToRad(x))
  #define cosdegr(x) cos(degrToRad(x))

  #define PARMSN 4
  #define PARMSZ 64

  #define EQUAT_RADIUS 6378.137  
  #define EQUAT_RADIUS_M 6378137  
  #define ECCENTRICITY 0.081082

  #define MAP_W 640
  #define MAP_H 512
  #define BASESCALE  20000000
  #define STATESCALE  5000000  
  #define ZONESCALE   1000000  
  #define CITYSCALE    150000  
  
  #define PIXELFACT 2817.947378
  #define KM2MILES 0.62137119
  #define KM2NAUTIC 0.54

  #define METRIC 0
  #define MILES  1
  #define NAUTIC 2

  #define N  0
  #define S  1
  #define E  2
  #define W  3
  #define NE 4
  #define SE 5
  #define SW 6
  #define NW 7

  #define CENTER 8
  #define RECT 9

  #define BORDER 20

  #define ZOOMIN  0
  #define ZOOMOUT 1

  #define SCALES 30
#endif

@class NSString;
@class NSMenuItem;

extern long scales[SCALES];

float radiusForLatitude(float lat);

NSPoint screenPointForEarthPoint(NSPoint ep, NSPoint cp, long scale);

NSPoint earthPointForScreenPoint(NSPoint sp, NSPoint cp, long scale);

BOOL earthPointFits(NSPoint ep, NSPoint cp, long scale);

NSString *coveredAreaForScale(long scale, int units);

float distanceBetweenPoints(NSPoint actp, NSPoint dstp);

float angleBetweenPoints(NSPoint actp, NSPoint dstp);

NSMenuItem *addItemToMenu(NSMenu *menu, NSString *str, 
														NSString *comm, NSString *sel, NSString *key);

#endif  // FUNCTIONS_H
