/* 
 * GPSInterface.h
 * 
 * Copyright (c) 2003 Enrico Sersale <enrico@imago.ro>
 *
 * Author: Enrico Sersale <enrico@imago.ro>
 * Date: August 2003
 *
 * This file is part of the GNUstep Position application
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef GPSINTERFACE_H
#define GPSINTERFACE_H

#include <Foundation/Foundation.h>

@class Position;

@interface NSFileHandle (NSFileHandleSocketConnection)

+ (id)fileHandleForConnectionToHost:(NSString *)hostname 
                             atPort:(unsigned)port;
@end

@interface GPSInterface: NSObject 
{  
  NSFileHandle *socketHandle;
  NSDate *timeoutDate;
  NSTimer *timeoutTimer;
  
  NSMutableString *nmeaSentence;
  NSNotificationCenter *nc;
  
  NSMutableArray *satellites;
  NSCalendarDate *date;
  NSDate *lastUpdate;
  int usedSats;  
  int visibleSats;
  BOOL satsDone;
  int status;
  int mode;
  float speed;
  float track;
  float latitude;
  float longitude;
  float altitude;
  float pdop;
  float hdop;
  float vdop;  
  
  Position *pos;
}

- (BOOL)openGpsdConnectionToHost:(NSString *)hostname 
                          atPort:(NSString *)portname
                        gpsdPath:(NSString *)gpsdpath
                        gpsdArgs:(NSString *)gpsdargs;
                              
- (void)checkConnection:(id)sender;

- (void)closeConnection;

- (void)socketData:(NSNotification *)notif;

- (NSString *)checkSentence;

- (NSString *)processSentence:(NSString *)sentence;

- (void)processGPRMC:(NSArray *)parts;

- (void)processGPGGA:(NSArray *)parts;

- (void)processGPGSA:(NSArray *)parts;

- (void)processGPGSV:(NSArray *)parts;

- (NSMutableDictionary *)satWithName:(NSString *)name;

- (NSMutableDictionary *)addSatWithName:(NSString *)name;

- (void)getLatitude:(float *)lat longitude:(float *)lon;

@end

#endif // GPSINTERFACE_H
