/* 
 * SatsInspector.h
 * 
 * Copyright (c) 2003 Enrico Sersale <enrico@imago.ro>
 *
 * Author: Enrico Sersale <enrico@imago.ro>
 * Date: August 2003
 *
 * This file is part of the GNUstep Position application
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifndef SATS_INSPECTOR_H
#define SATS_INSPECTOR_H

#include <Foundation/Foundation.h>
#include <AppKit/NSView.h>
#include "InspectorsProtocol.h"

@class NSColor;
@class NSImage;
@class SatsView;
@class SignalIndicator;
@class SatIndicator;

@interface SatsInspector : NSObject <InspectorsProtocol>
{
  IBOutlet id win;
  IBOutlet id mainBox;
  IBOutlet id mapBox;
  IBOutlet id indicatorsBox;
  
  SatsView *satsView;
  
  NSMutableArray *indicators;
  int usedSats;
}

- (void)setUsedSats:(NSNotification *)notif;

- (void)updateSatellites:(NSNotification *)notif;

- (SatIndicator *)indicatorWithName:(NSString *)iname;

@end

@interface SatsView : NSView
{
  NSImage *image;
  NSMutableArray *labels;
}

- (void)setElevation:(int)elevation 
             azimuth:(int)azimuth 
      forSatWithName:(NSString *)name
             atIndex:(int)index;

- (void)setStrength:(int)str forSatAtIndex:(int)index;

@end

@interface SatIndicator : NSObject
{
  IBOutlet id win;
  IBOutlet id mainBox;
  IBOutlet id indicatorBox;
  IBOutlet id label;
  SignalIndicator *signIndicator;
  float indw;
  float indorx;
  float indory;  
  NSString *satName;
  int strength;
  int lastStr;
}

- (void)setSatName:(NSString *)sname;

- (NSString *)satName;

- (void)setStrength:(int)str;

- (int)strength;

- (id)label;

- (id)mainBox;

@end

@interface SignalIndicator : NSView 
{
  NSColor *color;
}

@end

#endif // SATS_INSPECTOR_H
