/* 
 * MapDownload.m
 * 
 * Copyright (c) 2003 Enrico Sersale <enrico@imago.ro>
 *
 * Author: Enrico Sersale <enrico@imago.ro>
 * Date: August 2003
 *
 * This file is part of the GNUstep Position application
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <Foundation/Foundation.h>
#include <AppKit/AppKit.h>
#include "MapDownload.h"
#include "Functions.h"
#include "Notifications.h"
#include "Position.h"
#include "GNUstep.h"

#define MAXFILES 99999
#define BUFF_SIZE 2048

static NSString *nibName = @"DownloadMapWin";
static NSString *defServerName = @"www.vicinity.com";
static NSString *defFormatStr = @"http://%@/gif?&CT=%f:%f:%d&IC=&W=%i&H=%i&FAM=myblast";

@implementation MapDownload

- (void)dealloc
{
  [[NSNotificationCenter defaultCenter] removeObserver: self];
  TEST_RELEASE (win);
  TEST_RELEASE (serverName);
  TEST_RELEASE (formatString);
  TEST_RELEASE (tmpPath);
  TEST_RELEASE (mapsDir);
  TEST_RELEASE (fileName);
  TEST_RELEASE (urlHandle);
  TEST_RELEASE (fileHandle);
  
  [super dealloc];
}

- (id)init
{
  self = [super init];
  
  if (self) {
		if ([NSBundle loadNibNamed: nibName owner: self] == NO) {
      NSLog(@"failed to load %@!", nibName);
    } else {
	    NSUserDefaults *defaults;
      id entry;
      NSString *home;
      int i;
      
      fm = [NSFileManager defaultManager];
      pos = [Position position];    

      if ([win setFrameUsingName: @"mapdownload"] == NO) {
        [win setFrame: NSMakeRect(300, 300, 296, 283) display: NO];
      }
      [win setDelegate: self];  

      while ([[scalePopUp itemArray] count] > 0) {
        [scalePopUp removeItemAtIndex: 0];
      }
      for (i = 0; i < SCALES; i++) {
        NSNumber *num = [NSNumber numberWithLong: scales[i]];
        [scalePopUp addItemWithTitle: [num stringValue]]; 
      }

      [progressInd setMinValue: 0];
      [progressInd setMaxValue: 1000000];
      
      [self setScale: scalePopUp];

      defaults = [NSUserDefaults standardUserDefaults];
      entry = [defaults objectForKey: @"servername"];
      if (entry) {
        serverName = [[NSString alloc] initWithString: entry];
      } else {
        serverName = [[NSString alloc] initWithString: defServerName];
      }
      [serverField setStringValue: serverName];
      
      entry = [defaults objectForKey: @"formatstring"];
      if (entry) {
        formatString = [[NSString alloc] initWithString: entry];
      } else {
        formatString = [[NSString alloc] initWithString: defFormatStr];
      }

      entry = [defaults objectForKey: @"fileidentifier"];
      if (entry) {
        fileIdentifier = [entry intValue];
      } else {
        fileIdentifier = 0;
      }      

      home = [NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES) lastObject];
      ASSIGN (mapsDir, [home stringByAppendingPathComponent: @"Position"]);
      tmpPath = nil;
      fileName = nil;
      urlHandle = nil;
      fileHandle = nil;
      downloading = NO;

      [[NSNotificationCenter defaultCenter] addObserver: self 
                				selector: @selector(unitsDidChange:) 
                					  name: GPSDidChangeUnitsNotification
                					object: nil];
      
      /* Internationalization */
      [serverLabel setStringValue: NSLocalizedString(@"map server:", @"")];
      [latLabel setStringValue: NSLocalizedString(@"latitude:", @"")];
      [lonLabel setStringValue: NSLocalizedString(@"longitude:", @"")];
      [posExplLabel1 setStringValue: NSLocalizedString(@"You can also select the position", @"")];
      [posExplLabel2 setStringValue: NSLocalizedString(@"with a click on the map", @"")];
      [scaleLabel setStringValue: NSLocalizedString(@"scale:", @"")];
      [coversLabel setStringValue: NSLocalizedString(@"map covers:", @"")];
      [fnameLabel setStringValue: NSLocalizedString(@"file name:", @"")];
      [cancelButt setTitle: NSLocalizedString(@"Cancel", @"")];
      [downloadButt setTitle: NSLocalizedString(@"Download", @"")];
    }
  }
  
  return self;
}

- (void)activateWithInfo:(NSDictionary *)info
{
  id entry;
  
  entry = [info objectForKey: @"longitude"];
  [lonField setStringValue: [entry stringValue]];
  entry = [info objectForKey: @"latitude"];
  [latField setStringValue: [entry stringValue]];
  entry = [info objectForKey: @"scale"];
  [scalePopUp selectItemWithTitle: [entry stringValue]];
  
  [self updateCoveredArea];
  
	fileIdentifier++;
	if (fileIdentifier >= MAXFILES) {
		fileIdentifier = 0;
	}
  [fnameField setStringValue: [NSString stringWithFormat: @"map-%i.gif", fileIdentifier]];
  
  [win makeKeyAndOrderFront: nil];
}

- (void)updateCoveredArea
{
  long scale = [[scalePopUp titleOfSelectedItem] intValue];

  [coversField setStringValue: coveredAreaForScale(scale, [pos units])];
}

- (void)updateLongitude:(float)lon latitude:(float)lat
{
  NSString *str = [NSString stringWithFormat: @"%f", lon];
  [lonField setStringValue: str];
  str = [NSString stringWithFormat: @"%f", lat];
  [latField setStringValue: str];
}

- (IBAction)setScale:(id)sender
{
  [self updateCoveredArea];
}

- (IBAction)startDownload:(id)sender
{
  float lon = [lonField floatValue];
  float lat = [latField floatValue];
  long scale = [[scalePopUp titleOfSelectedItem] intValue];
  NSString *fname = [fnameField stringValue];
  
  if (lon == 0 || lat == 0 || [fname length] == 0) {
    NSRunAlertPanel(NSLocalizedString(@"error", @""),
                    NSLocalizedString(@"Invalid value!", @""), 
                              NSLocalizedString(@"OK", @""), nil, nil); 
    return;
  } else {
    NSString *str;
    NSURL *url;
    Class class;
    
    str = [NSString stringWithFormat: formatString, 
                                serverName, lat, lon, scale, MAP_W, MAP_H];
    
    url = [NSURL URLWithString: str];
    if (url == nil) {
      NSRunAlertPanel(NSLocalizedString(@"error", @""),
                    NSLocalizedString(@"Bad url!", @""), 
                              NSLocalizedString(@"OK", @""), nil, nil); 
      return;
    }
    
    class = [NSURLHandle URLHandleClassForURL: url];
    urlHandle = [[class alloc] initWithURL: url cached: NO];
    [urlHandle addClient: self];
    
    ASSIGN (tmpPath, [NSTemporaryDirectory() stringByAppendingPathComponent: fname]);  
  
    [fm createFileAtPath: tmpPath contents: nil attributes: nil];
    
    fileHandle = [NSFileHandle fileHandleForWritingAtPath: tmpPath];
    RETAIN (fileHandle);
  
    fileLength = 0;
    
    [urlHandle loadInBackground];

	  fileIdentifier++;
	  if (fileIdentifier >= MAXFILES) {
		  fileIdentifier = 0;
	  }
    [fnameField setStringValue: [NSString stringWithFormat: @"map-%i.gif", fileIdentifier]];
  } 
}

- (IBAction)stopDownload:(id)sender
{
  if (downloading) {
    [urlHandle cancelLoadInBackground];
  }
}

- (void)unitsDidChange:(NSNotification *)notif
{
  [self updateCoveredArea];
}

- (void)updateDefaults
{
  NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

  [defaults setObject: [NSString stringWithFormat: @"%i", fileIdentifier]
               forKey: @"fileidentifier"];
  [defaults synchronize];

  [win saveFrameUsingName: @"mapdownload"];
  
  if (downloading) {
    [self stopDownload: nil];
  }
}

- (id)myWin
{
  return win;
}

- (BOOL)windowShouldClose:(id)sender
{
  [self updateDefaults];
	return YES;
}

- (void)URLHandleResourceDidBeginLoading:(NSURLHandle *)sender
{
}

- (void)URLHandleResourceDidCancelLoading:(NSURLHandle *)sender
{
  [fileHandle closeFile];
  [progressInd setMinValue: 0];
  [progressInd setMaxValue: 1000000];
  
  NSRunAlertPanel(NSLocalizedString(@"error", @""), 
              NSLocalizedString(@"Download cancelled", @""), 
                            NSLocalizedString(@"OK", @""), nil, nil); 
}

- (void)URLHandleResourceDidFinishLoading:(NSURLHandle *)sender
{
  NSMutableDictionary *info = [NSMutableDictionary dictionary];

  [fileHandle closeFile];
  [progressInd setMinValue: 0];
  [progressInd setMaxValue: 1000000];
  
  NSLog(@"URLHandleResourceDidFinishLoading");
  
  [info setObject: tmpPath forKey: @"path"];
  [info setObject: [lonField stringValue] forKey: @"longitude"];
  [info setObject: [latField stringValue] forKey: @"latitude"];
  [info setObject: [scalePopUp titleOfSelectedItem] forKey: @"scale"];

  [pos showMapImportForDownloadedFile: info];
}

- (void)URLHandle:(NSURLHandle *)sender resourceDataDidBecomeAvailable:(NSData *)newBytes
{
  if (fileLength == 0) {
    NSString *contentLength = [sender propertyForKeyIfAvailable: @"Content-Length"];

    if (contentLength) {
      fileLength = [contentLength intValue];
      [progressInd setMinValue: 0];
      [progressInd setMaxValue: fileLength / BUFF_SIZE];
    }
  } else {
    [progressInd incrementBy: 1.0];
  }

  [fileHandle writeData: newBytes];
}

- (void)URLHandle:(NSURLHandle *)sender resourceDidFailLoadingWithReason:(NSString *)reason
{
  [fileHandle closeFile];
  [progressInd setMinValue: 0];
  [progressInd setMaxValue: 1000000];

  NSRunAlertPanel(NSLocalizedString(@"error", @""), reason, 
                            NSLocalizedString(@"OK", @""), nil, nil); 
}

@end

