/* 
 * MapImport.m
 * 
 * Copyright (c) 2003 Enrico Sersale <enrico@imago.ro>
 *
 * Author: Enrico Sersale <enrico@imago.ro>
 * Date: August 2003
 *
 * This file is part of the GNUstep Position application
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <Foundation/Foundation.h>
#include <AppKit/AppKit.h>
#include "MapImport.h"
#include "Functions.h"
#include "Notifications.h"
#include "Position.h"
#include "GNUstep.h"

static NSString *nibName = @"ImportMapWin";

@implementation MapImport

- (void)dealloc
{
  [[NSNotificationCenter defaultCenter] removeObserver: self];
  TEST_RELEASE (win);
  TEST_RELEASE (imagePath);
  
  [super dealloc];
}

- (id)init
{
  self = [super init];
  
  if (self) {
		if ([NSBundle loadNibNamed: nibName owner: self] == NO) {
      NSLog(@"failed to load %@!", nibName);
    } else {      
      fm = [NSFileManager defaultManager];
      pos = [Position position]; 

      if ([win setFrameUsingName: @"mapimport"] == NO) {
        [win setFrame: NSMakeRect(300, 300, 417, 294) display: NO];
      }
      [win setDelegate: self];  

      [[NSNotificationCenter defaultCenter] addObserver: self 
                				selector: @selector(unitsDidChange:) 
                					  name: GPSDidChangeUnitsNotification
                					object: nil];
      
      /* Internationalization */
      [widthLabel setStringValue: NSLocalizedString(@"width:", @"")];
      [heightLabel setStringValue: NSLocalizedString(@"height:", @"")];
      [lonLabel setStringValue: NSLocalizedString(@"longitude:", @"")];
      [latLabel setStringValue: NSLocalizedString(@"latitude:", @"")];
      [scaleLabel setStringValue: NSLocalizedString(@"scale:", @"")];
      [coversLabel setStringValue: NSLocalizedString(@"covers:", @"")];
      [chooseButt setTitle: NSLocalizedString(@"Choose", @"")];
      [importButt setTitle: NSLocalizedString(@"Import", @"")];
    }
  }
  
  return self;
}

- (void)activate
{
  [win makeKeyAndOrderFront: nil];
}

- (void)setFromLocalBehaviour
{
  [lonField setEditable: YES];
  [lonField setSelectable: YES];
  [lonField setBezeled: YES];
  [lonField setDrawsBackground: YES];
  [lonField setStringValue: @""];
  
  [latField setEditable: YES];
  [latField setSelectable: YES];
  [latField setBezeled: YES];
  [latField setDrawsBackground: YES];
  [latField setStringValue: @""];

  [scaleField setEditable: YES];
  [scaleField setSelectable: YES];
  [scaleField setBezeled: YES];
  [scaleField setDrawsBackground: YES];
  [scaleField setStringValue: @""];

  [widthField setStringValue: @""];
  [heightField setStringValue: @""];
  [lonField setStringValue: @""];
  [latField setStringValue: @""];
  [scaleField setStringValue: @""];
  [coversField setStringValue: @""];

  [chooseButt setEnabled: YES];
  [importButt setEnabled: NO];
  
  [imageView setImage: nil];
}

- (void)setFromNetBehaviour
{
  [lonField setEditable: NO];
  [lonField setSelectable: NO];
  [lonField setBezeled: NO];
  [lonField setDrawsBackground: NO];
  [lonField setStringValue: @""];

  [latField setEditable: NO];
  [latField setSelectable: NO];
  [latField setBezeled: NO];
  [latField setDrawsBackground: NO];
  [latField setStringValue: @""];

  [scaleField setEditable: NO];
  [scaleField setSelectable: NO];
  [scaleField setBezeled: NO];
  [scaleField setDrawsBackground: NO];
  [scaleField setStringValue: @""];

  [widthField setStringValue: @""];
  [heightField setStringValue: @""];
  [lonField setStringValue: @""];
  [latField setStringValue: @""];
  [scaleField setStringValue: @""];
  [coversField setStringValue: @""];

  [chooseButt setEnabled: NO];
  [importButt setEnabled: NO];
  
  [imageView setImage: nil];
}

- (void)prepareToImportDownloadedFile:(NSDictionary *)fileInfo
{
  NSImage *image;

  ASSIGN (imagePath, [fileInfo objectForKey: @"path"]);
  image = [[NSImage alloc] initWithContentsOfFile: imagePath];

  if (image == nil) {
    NSRunAlertPanel(NSLocalizedString(@"error", @""),
                    NSLocalizedString(@"Bad image file!", @""), 
                              NSLocalizedString(@"OK", @""), nil, nil); 
    return;
  } else {
    NSString *lonStr = [fileInfo objectForKey: @"longitude"];
    NSString *latStr = [fileInfo objectForKey: @"latitude"];
    NSString *scaleStr = [fileInfo objectForKey: @"scale"];
    long scale = [scaleStr intValue];
    NSSize size = [image size];
    
    [imageView setImage: image];
    RELEASE (image);

    [widthField setStringValue: [NSString stringWithFormat: @"%f", size.width]];
    [heightField setStringValue: [NSString stringWithFormat: @"%f", size.height]];
    [lonField setStringValue: lonStr];
    [latField setStringValue: latStr];
    [scaleField setStringValue: scaleStr];
    [coversField setStringValue: coveredAreaForScale(scale, [pos units])];
    
    [importButt setEnabled: YES];
  }
}

- (IBAction)choseMapFile:(id)sender
{
	NSOpenPanel *openPanel;
	NSArray *fileTypes;
  NSImage *image;
	int result;
   
	fileTypes = [NSArray arrayWithObjects: @"tiff", @"tif", @"jpeg", @"jpg", @"gif", @"png", nil];
  
	openPanel = [NSOpenPanel openPanel];
	[openPanel setTitle: @"choose map image"];	
  [openPanel setAllowsMultipleSelection: NO];
  [openPanel setCanChooseFiles: YES];
  [openPanel setCanChooseDirectories: NO];

  result = [openPanel runModalForDirectory: NSHomeDirectory() file: nil types: fileTypes];
	if(result != NSOKButton) {
		return;
  }
  
  ASSIGN (imagePath, [openPanel filename]);
  image = [[NSImage alloc] initWithContentsOfFile: imagePath];

  if (image == nil) {
    NSRunAlertPanel(NSLocalizedString(@"error", @""),
                    NSLocalizedString(@"Bad image file!", @""), 
                              NSLocalizedString(@"OK", @""), nil, nil); 
    return;
  } else {
    NSSize size = [image size];

    if (size.width != MAP_W || size.height != MAP_H) {
      NSRunAlertPanel(NSLocalizedString(@"error", @""),
                    NSLocalizedString(@"Invalid image size!", @""), 
                              NSLocalizedString(@"OK", @""), nil, nil); 
      return;
    }
    
    [imageView setImage: image];
    RELEASE (image);

    [widthField setStringValue: [NSString stringWithFormat: @"%f", size.width]];
    [heightField setStringValue: [NSString stringWithFormat: @"%f", size.height]];

    [importButt setEnabled: YES];
  }
}

- (IBAction)saveMapFile:(id)sender
{
  NSString *lonStr = [lonField stringValue];
  NSString *latStr = [latField stringValue];
  NSString *scaleStr = [scaleField stringValue];

  if ([pos addMapFromFile: imagePath longitude: [lonStr floatValue]
                 latitude: [latStr floatValue] scale: [scaleStr intValue]]) {
    if ([[imagePath stringByDeletingLastPathComponent] isEqual: NSTemporaryDirectory()]) {
      [fm removeFileAtPath: imagePath handler: nil];
    }
    
    [importButt setEnabled: NO];
  }
}

- (void)unitsDidChange:(NSNotification *)notif
{
}

- (void)updateDefaults
{
  [win saveFrameUsingName: @"mapimport"];
}

- (id)myWin
{
  return win;
}

- (BOOL)windowShouldClose:(id)sender
{
  [self updateDefaults];
	return YES;
}

@end





