/* 
 * MapWindow.m
 * 
 * Copyright (c) 2003 Enrico Sersale <enrico@imago.ro>
 *
 * Author: Enrico Sersale <enrico@imago.ro>
 * Date: August 2003
 *
 * This file is part of the GNUstep Position application
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <Foundation/Foundation.h>
#include <AppKit/AppKit.h>
#include "MapWindow.h"
#include "MapView.h"
#include "GNUstep.h"

static NSString *nibName = @"MapWindow";

@implementation MapWindow

- (void)dealloc
{ 
  RELEASE (win);    
  [super dealloc];	
}

- (id)init
{  
	self = [super init];

  if (self) {
		if ([NSBundle loadNibNamed: nibName owner: self] == NO) {
      NSLog(@"Preferences: failed to load %@!", nibName);
    } 
  }
    
	return self;
}

- (void)awakeFromNib
{
  if ([win setFrameUsingName: @"mapwindow"] == NO) {
    [win setFrame: NSMakeRect(100, 100, 640, 534) display: NO];
  }
  [win setDelegate: self];  

  mapView = [[MapView alloc] init];
  [mapView setFrame: [[win contentView] frame]];
  [[win contentView] addSubview: mapView];
  RELEASE (mapView); 

  /*  Internationalization */
  [win setTitle: NSLocalizedString(@"Map", @"")];
}

- (void)activate
{
  [win makeKeyAndOrderFront: nil];
}

- (id)myWin
{
  return win;
}

- (MapView *)mapView
{
  return mapView;
}

- (void)saveDefaults
{
  [win saveFrameUsingName: @"mapwindow"];
}

- (BOOL)windowShouldClose:(id)sender
{
  [win saveFrameUsingName: @"preferencesWin"];
	return YES;
}

@end
