/* 
 * Position.h
 * 
 * Copyright (c) 2003 Enrico Sersale <enrico@imago.ro>
 *
 * Author: Enrico Sersale <enrico@imago.ro>
 * Date: August 2003
 *
 * This file is part of the GNUstep Position application
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef POSITION_H
#define POSITION_H

@class NSMutableArray;
@class NSMutableDictionary;
@class NSCalendarDate;
@class NSTimer;
@class MapWindow;
@class Map;
@class GPSInterface;
@class InspectorsController;
@class PrefController;
@class MapDownload;
@class MapImport;
@class WayPoint;

@interface Position : NSObject 
{
  MapWindow *mapWindow;
  GPSInterface *gpsint;
  NSString *hostName;
  NSString *hostPort;
  NSString *gpsdPath;
  NSString *gpsdArgs;
  
  NSString *mapsDir;
  NSString *mapsDictPath;
  NSMutableArray *maps;
  Map *currentMap;
  NSString *currMapPath;
  int scaleIndex;
    
  NSString *wpointsPath;
  NSMutableDictionary *wayPointsDict;
  
  NSString *routesPath;
  NSMutableDictionary *routesDict;
  NSArray *followedRoute;
  NSString *follRouteName;
  int followPntInd;
  float nextPntDist;
  float nextPointDir;
  BOOL nearTarget;
  BOOL followRoute;
  BOOL voiceAlerts;
  BOOL alertDone;
  
  float longitude;
  float latitude;
  float lonCorr;
  float latCorr;
  BOOL useTuning;
  
  int units;
  
  InspectorsController *inspController;
  PrefController *prefController;  
  MapDownload *mapDownload;
  MapImport *mapImport;
}

+ (id)position;

+ (void)registerForServices;

- (void)createGpsInterface;

- (void)gpsdConnectionLost;

- (void)createMapWindow;

- (BOOL)addMapFromFile:(NSString *)path 
             longitude:(float)lon
              latitude:(float)lat
                 scale:(long)scale;

- (void)setDefaultMapPath:(NSString *)mpath 
                 withInfo:(NSDictionary *)mapinfo;

- (NSString *)defaultMapPath;

- (void)setLongitude:(float)lot latitude:(float)lat;

- (void)setAltitude:(float)alt;

- (void)setSpeed:(float)speed;

- (void)setTrack:(float)track;

- (void)setGpsDate:(NSCalendarDate *)date;

- (void)setUsedSats:(int)satsnum;

- (void)updateSatellites:(NSArray *)satsArr;

- (void)clickOnMap:(NSPoint)p;

- (void)shouldChangeMap:(int)direction;

- (void)zoom:(int)d;

- (void)zoomIn:(id)sender;

- (void)zoomOut:(id)sender;

- (Map *)mapWithPath:(NSString *)mpath;

- (NSArray *)mapsWithScale:(long)sc;

- (void)tuneLongitude:(float)value;

- (float)longitudeTune;

- (void)tuneLatitude:(float)value;

- (float)latitudeTune;

- (void)setUseTuning:(BOOL)value;

- (BOOL)useTuning;

- (void)setUnits:(int)value;

- (int)units;

- (void)connSettingsdidChangeHost:(NSString *)hostname 
                             port:(NSString *)portname
                         gpsdPath:(NSString *)gpsdpath
                         gpsdArgs:(NSString *)gpsdargs;

- (void)updateDefaults;

- (void)connectToGpsdHost:(id)sender;

- (void)disconnectFromGpsdHost:(id)sender;

- (void)showInspectors:(id)sender;

- (void)showSatInspector:(id)sender;

- (void)showNavigationInspector:(id)sender;

- (void)showWPointInspector:(id)sender;

- (void)showRouteInspector:(id)sender;

- (void)showPreferences:(id)sender;

- (void)showMapDownload:(id)sender;

- (void)showMapImport:(id)sender;

- (void)showMapImportForDownloadedFile:(NSDictionary *)fileInfo;

- (void)showInfo:(id)sender;

#ifndef GNUSTEP
- (void)terminate:(id)sender;
#endif

@end

@interface Position (WayPoints)

- (void)createWayPoint:(id)sender;

- (void)deleteWayPoint:(id)sender;

- (void)updateDictionaryForWayPoint:(WayPoint *)wpoint 
                            oldName:(NSString *)oldname;

- (void)changedSelectedWayPoint:(WayPoint *)wpoint;

- (WayPoint *)selectedWayPoint;

- (WayPoint *)wayPointWithName:(NSString *)wname;

- (NSArray *)wayPointsForCenterPoint:(NSPoint)cp atScale:(long)scale;

- (NSArray *)wayPointIconsNames;

@end

@interface Position (Routes)

- (NSDictionary *)routesDict;

- (void)addRouteWithName:(NSString *)rname;

- (void)removeRouteWithName:(NSString *)rname;

- (void)addWayPoint:(WayPoint *)wpoint toRouteWithName:(NSString *)rname;

- (void)addWayPoints:(NSArray *)points toRouteWithName:(NSString *)rname;

- (void)removeWayPointWithName:(NSString *)wpname
               inRouteWithName:(NSString *)rname;

- (NSArray *)pointsOfRouteWithName:(NSString *)rname;

- (NSDictionary *)infoOfWayPointAtIndex:(int)index;

- (void)followRouteWithName:(NSString *)rname;

- (void)stopFollowRoute;

- (void)nextPointInRoute:(id)sender;

- (BOOL)followRoute;

- (NSArray *)followedRoute;
  
- (NSString *)follRouteName;

- (void)distanceFromNextPoint;

- (void)sayAlert:(NSString *)alert;

- (void)setSayAlerts:(BOOL)value;

@end

#endif // POSITION_H
