/* 
 * GPSDHostPref.m
 * 
 * Copyright (c) 2003 Enrico Sersale <enrico@imago.ro>
 *
 * Author: Enrico Sersale <enrico@imago.ro>
 * Date: August 2003
 *
 * This file is part of the GNUstep Position application
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <Foundation/Foundation.h>
#include <AppKit/AppKit.h>
#include "GPSDHostPref.h"
#include "Functions.h"
#include "Position.h"
#include "GNUstep.h"

#ifndef HOST_NAME
  #define HOST_NAME @"localhost"
#endif
#ifndef HOST_PORT
  #define HOST_PORT @"2947"
#endif
#ifndef GPSD_PATH
  #define GPSD_PATH @"/usr/local/bin/gpsd"
#endif
#ifndef GPSD_ARGS
  #define GPSD_ARGS @"-p /dev/ttyS0"
#endif

static NSString *nibName = @"GPSDHostPref";

@implementation GPSDHostPref

- (void)dealloc
{
  TEST_RELEASE (prefbox);
  [super dealloc];
}

- (id)init
{
  self = [super init];
  
  if (self) {
		if ([NSBundle loadNibNamed: nibName owner: self] == NO) {
      NSLog(@"failed to load %@!", nibName);
    } else {
 	    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
      id entry;
      
      RETAIN (prefbox);
      RELEASE (win); 

      pos = [Position position];    
      
      entry = [defaults objectForKey: @"hostname"];
      if (entry) {
        [hostField setStringValue: entry];
      } else {
        [hostField setStringValue: HOST_NAME];
      }

      entry = [defaults objectForKey: @"hostport"];
      if (entry) {
        [portField setStringValue: entry];
      } else {
        [portField setStringValue: HOST_PORT];
      }

      entry = [defaults objectForKey: @"gpsdpath"];
      if (entry) {
        [gpsdPathField setStringValue: entry];
      } else {
        [gpsdPathField setStringValue: GPSD_PATH];
      }

      entry = [defaults objectForKey: @"gpsdargs"];
      if (entry) {
        [gpsdArgsField setStringValue: entry];
      } else {
        [gpsdArgsField setStringValue: GPSD_ARGS];
      }

      /* Internationalization */
      [fieldsBox setTitle: NSLocalizedString(@"gpsd host settings", @"")];
      [hostLabel setStringValue: NSLocalizedString(@"host", @"")];
      [portLabel setStringValue: NSLocalizedString(@"port", @"")];
      [localFldsBox setTitle: NSLocalizedString(@"local gpsd settings", @"")];
      [gpsdPathLabel setStringValue: NSLocalizedString(@"gpsd path", @"")];
      [gpsdArgsLabel setStringValue: NSLocalizedString(@"gpsd arguments", @"")];
    }
  }
  
  return self;
}

- (IBAction)applyButtAction:(id)sender
{
  NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
  NSString *hostStr = [hostField stringValue];
  NSString *portStr = [portField stringValue];
  NSString *gpsdPath = [gpsdPathField stringValue];
  NSString *gpsdArgs = [gpsdArgsField stringValue];

  if([hostStr length]) {
    [defaults setObject: hostStr forKey: @"hostname"];
  }

  if([portStr length]) {
    [defaults setObject: portStr forKey: @"hostport"];
  }

  if([gpsdPath length]) {
    [defaults setObject: gpsdPath forKey: @"gpsdpath"];
  }

  if([gpsdArgs length]) {
    [defaults setObject: gpsdArgs forKey: @"gpsdargs"];
  }

  [defaults synchronize];
  
  [pos connSettingsdidChangeHost: hostStr 
                            port: portStr 
                        gpsdPath: gpsdPath 
                        gpsdArgs: gpsdArgs];
}

- (NSView *)prefView
{
  return prefbox;
}

- (NSString *)prefName
{
  return NSLocalizedString(@"gpsd host", @"");
}

@end
