/* 
 * GeneralPref.m
 * 
 * Copyright (c) 2003 Enrico Sersale <enrico@imago.ro>
 *
 * Author: Enrico Sersale <enrico@imago.ro>
 * Date: August 2003
 *
 * This file is part of the GNUstep Position application
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <Foundation/Foundation.h>
#include <AppKit/AppKit.h>
#include "GeneralPref.h"
#include "Functions.h"
#include "Position.h"
#include "GNUstep.h"

static NSString *nibName = @"GeneralPref";

@implementation GeneralPref

- (void)dealloc
{
  TEST_RELEASE (prefbox);
  [super dealloc];
}

- (id)init
{
  self = [super init];
  
  if (self) {
		if ([NSBundle loadNibNamed: nibName owner: self] == NO) {
      NSLog(@"failed to load %@!", nibName);
    } else {
 	    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
      
      RETAIN (prefbox);
      RELEASE (win); 

      pos = [Position position];  
      
      if ([defaults boolForKey: @"sayalerts"]) {
        [speakSwitch setState: NSOnState];
      } else {
        [speakSwitch setState: NSOffState];
      }
      
      /* Internationalization */
      [fieldsBox setTitle: NSLocalizedString(@"General settings", @"")];
      [speakSwitch setStringValue: NSLocalizedString(@"say alerts", @"")];
    }
  }
  
  return self;
}

- (IBAction)speakSwitchAction:(id)sender
{
  NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
  
  if ([sender state] == NSOnState) {
    [defaults setBool: YES forKey: @"sayalerts"];
    [pos setSayAlerts: YES];
  } else {
    [defaults setBool: NO forKey: @"sayalerts"]; 
    [pos setSayAlerts: NO];
  }
}

- (NSView *)prefView
{
  return prefbox;
}

- (NSString *)prefName
{
  return NSLocalizedString(@"General", @"");
}

@end
