/* 
 * UnitsAndTuningPref.m
 * 
 * Copyright (c) 2003 Enrico Sersale <enrico@imago.ro>
 *
 * Author: Enrico Sersale <enrico@imago.ro>
 * Date: August 2003
 *
 * This file is part of the GNUstep Position application
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <Foundation/Foundation.h>
#include <AppKit/AppKit.h>
#include "UnitsAndTuningPref.h"
#include "Functions.h"
#include "Position.h"
#include "GNUstep.h"

static NSString *nibName = @"UnitsAndTuningPref";

@implementation UnitsAndTuningPref

- (void)dealloc
{
  TEST_RELEASE (prefbox);

  [super dealloc];
}

- (id)init
{
  self = [super init];
  
  if (self) {
		if ([NSBundle loadNibNamed: nibName owner: self] == NO) {
      NSLog(@"failed to load %@!", nibName);
    } else {
      RETAIN (prefbox);
      RELEASE (win); 

      pos = [Position position];   
      
      [useTuningCheck setState: ([pos useTuning] ? NSOnState : NSOffState)];
   
      [lonStepper setFloatValue: [pos longitudeTune]];
      [lonField setStringValue: [NSString stringWithFormat: @"%f", [pos longitudeTune]]];
      [latStepper setFloatValue: [pos latitudeTune]];
      [latField setStringValue: [NSString stringWithFormat: @"%f", [pos latitudeTune]]];

      [unitsMatrix selectCellAtRow: [pos units] column: 0];

      /* Internationalization */
      [tuningBox setTitle: NSLocalizedString(@"Tuning", @"")];
      [lonLabel setStringValue: NSLocalizedString(@"longitude", @"")];
      [latLabel setStringValue: NSLocalizedString(@"latitude", @"")];
      [useTuningCheck setTitle: NSLocalizedString(@"use tuning", @"")];
      [unitsBox setTitle: NSLocalizedString(@"Units", @"")];
      [[[unitsMatrix cells] objectAtIndex: 0] setStringValue: NSLocalizedString(@"metric", @"")];
      [[[unitsMatrix cells] objectAtIndex: 1] setStringValue: NSLocalizedString(@"miles", @"")];
      [[[unitsMatrix cells] objectAtIndex: 2] setStringValue: NSLocalizedString(@"nautic", @"")];
    }
  }
  
  return self;
}

- (IBAction)lonStepperAction:(id)sender
{
  float tunef = [sender floatValue];
  NSString *tuneStr = [NSString stringWithFormat: @"%f", tunef];

  [lonField setStringValue: tuneStr];
  [pos tuneLongitude: tunef];
}

- (IBAction)latStepperAction:(id)sender
{
  float tunef = [sender floatValue];
  NSString *tuneStr = [NSString stringWithFormat: @"%f", tunef];

  [latField setStringValue: tuneStr];
  [pos tuneLatitude: tunef];
}

- (IBAction)useTuningAction:(id)sender
{
  [pos setUseTuning: (([sender state] == NSOnState) ? YES : NO)];
}

- (void)matrixAction:(id)sender
{
  int row, col;
  
  [unitsMatrix getRow: &row column: &col ofCell: [unitsMatrix selectedCell]];
  [pos setUnits: row];
}

- (NSView *)prefView
{
  return prefbox;
}

- (NSString *)prefName
{
  return NSLocalizedString(@"Tuning & Units", @"");
}

@end
