/* 
 * WayPoint.m
 * 
 * Copyright (c) 2003 Enrico Sersale <enrico@imago.ro>
 *
 * Author: Enrico Sersale <enrico@imago.ro>
 * Date: August 2003
 *
 * This file is part of the GNUstep Position application
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <Foundation/Foundation.h>
#include <AppKit/AppKit.h>
#include "WayPoint.h"
#include "MapView.h"
#include "Position.h"
#include "Functions.h"
#include "GNUstep.h"

#define ICONPOSITION(s1, s2) (NSMakePoint(((int)(s1).width - (int)(s2).width) >> 1, \
((int)(s1).height - (int)(s2).height) >> 1))

@implementation WayPoint

- (void)dealloc
{ 
  RELEASE (name);   
  RELEASE (description);   
  RELEASE (iconName);   
  RELEASE (icon);   
  RELEASE (highlight);   
  RELEASE (label);
  
  [super dealloc];	
}

- (id)initWithDictionary:(NSDictionary *)dict
{
	self = [super init];

  if (self) {
    ASSIGNCOPY (name, [dict objectForKey: @"name"]);
    ASSIGNCOPY (description, [dict objectForKey: @"description"]);
    ASSIGNCOPY (iconName, [dict objectForKey: @"iconname"]);
    ASSIGN (icon, [NSImage imageNamed: iconName]);
    ASSIGN (highlight, [NSImage imageNamed: @"wayPointHiLight.tiff"]);

    longitude = [[dict objectForKey: @"longitude"] floatValue];
    latitude = [[dict objectForKey: @"latitude"] floatValue];
    scale = [[dict objectForKey: @"scale"] longValue];

    label = [NSTextField new];
    [label setDelegate: self];
		[label setFont: [NSFont systemFontOfSize: 12]];
		[label setBezeled: NO];
		[label setSelectable: NO];
		[label setEditable: NO];
		[label setAlignment: NSCenterTextAlignment];
	  [label setBackgroundColor: [NSColor whiteColor]];
	  [label setStringValue: name];
    [label setFrame: NSMakeRect(0, 0, [[label font] widthOfString: name] + 8, 14)];  
    
    pos = [Position position];   
    
    isSelect = NO;
  }
  
  return self;
}

- (void)setWaiPointInfo:(NSDictionary *)info
{
  MapView *mview = (MapView *)[self superview];
  NSString *oldname = [[NSString alloc] initWithString: name];
  NSPoint p = [label frame].origin;
  float lwidth;

  longitude = [[info objectForKey: @"longitude"] floatValue];
  latitude = [[info objectForKey: @"latitude"] floatValue];
  scale = [[info objectForKey: @"scale"] longValue];
  
  ASSIGNCOPY (name, [info objectForKey: @"name"]);
  ASSIGNCOPY (description, [info objectForKey: @"description"]);
  ASSIGNCOPY (iconName, [info objectForKey: @"iconname"]);
  ASSIGN (icon, [NSImage imageNamed: iconName]);

  [label setStringValue: name];
  lwidth = [[NSFont systemFontOfSize: 12] widthOfString: name] + 8;
  [mview setNeedsDisplayInRect: [label frame]];
	[label setFrame: NSMakeRect(p.x, p.y, lwidth, 14)];
  [mview setLabelFrameOfWayPoint: self];    

  [pos updateDictionaryForWayPoint: self oldName: oldname];

  RELEASE (oldname);
  
  [self setNeedsDisplay: YES];  
}

- (NSString *)iconName
{
  return iconName;
}

- (NSString *)name
{
  return name;
}

- (NSString *)description
{
  return description;
}

- (float)longitude
{
  return longitude;
}

- (float)latitude
{
  return latitude;
}

- (long)scale
{
  return scale;
}

- (NSTextField *)label
{
  return label;
}

- (void)select
{
  isSelect = YES;      
  [label setSelectable: YES];
  [label setEditable: YES];
  [(MapView *)[self superview] unselectOtherPoints: self];  
  [self setNeedsDisplay: YES];
  [pos changedSelectedWayPoint: self];
}

- (void)unselect
{
  isSelect = NO;
  [label setSelectable: NO];
  [label setEditable: NO];
  [self setNeedsDisplay: YES];
}

- (BOOL)isSelect
{
  return isSelect;
}

- (void)controlTextDidChange:(NSNotification *)aNotification
{
  MapView *mview = (MapView *)[self superview];
  NSDictionary *info = [aNotification userInfo];
  NSText *text = [info objectForKey: @"NSFieldEditor"];  
  NSString *current = [text string];
	NSPoint p = [label frame].origin;
  float lwidth = [[NSFont systemFontOfSize: 12] widthOfString: current] + 8;

  [mview setNeedsDisplayInRect: [label frame]];
	[label setFrame: NSMakeRect(p.x, p.y, lwidth, 14)];
  [mview setLabelFrameOfWayPoint: self];
}

- (void)controlTextDidEndEditing:(NSNotification *)aNotification
{
  MapView *mview = (MapView *)[self superview];
  NSString *oldname = [[NSString alloc] initWithString: name];
  NSString *newname = [label stringValue];
    
  ASSIGN (name, newname);
  [mview setNeedsDisplayInRect: [label frame]];
  [mview setLabelFrameOfWayPoint: self];

  [pos updateDictionaryForWayPoint: self oldName: oldname];
  
  RELEASE (oldname);
}

- (void)mouseDown:(NSEvent *)theEvent
{
  [self select];
}

- (void)drawRect:(NSRect)rect
{
	if(isSelect) {
		[highlight compositeToPoint: ICONPOSITION(rect.size, [highlight size]) 
                      operation: NSCompositeSourceOver];
	}

  [icon compositeToPoint: ICONPOSITION(rect.size, [icon size]) 
               operation: NSCompositeSourceOver];
}

- (BOOL)acceptsFirstResponder
{
  return YES;
}

- (BOOL)acceptsFirstMouse:(NSEvent *)theEvent
{
	return YES;
}

@end
