/* 
 * WayPoints.h
 * 
 * Copyright (c) 2003 Enrico Sersale <enrico@imago.ro>
 *
 * Author: Enrico Sersale <enrico@imago.ro>
 * Date: August 2003
 *
 * This file is part of the GNUstep Position application
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <Foundation/Foundation.h>
#include <AppKit/AppKit.h>
#include "Position.h"
#include "GPSInterface.h"
#include "MapWindow.h"
#include "MapView.h"
#include "Map.h"
#include "WayPoint.h"
#include "MapDownload.h"
#include "MapImport.h"
#include "Inspectors/InspectorsController.h"
#include "Preferences/PrefController.h"
#include "Functions.h"
#include "Notifications.h"
#include "GNUstep.h"
#include <math.h>

static NSString *defIconName = @"point.tiff";
static NSString *defDescription = @"waypoint";

@implementation Position (WayPoints)

- (void)createWayPoint:(id)sender
{
  NSMutableDictionary *dict = [NSMutableDictionary dictionary];

  [dict setObject: @"untitled" forKey: @"name"];
  [dict setObject: defDescription forKey: @"description"];
  [dict setObject: defIconName forKey: @"iconname"];
  [dict setObject: [NSNumber numberWithFloat: longitude] forKey: @"longitude"];
  [dict setObject: [NSNumber numberWithFloat: latitude] forKey: @"latitude"];
  [dict setObject: [NSNumber numberWithLong: scales[scaleIndex]] forKey: @"scale"];

  [wayPointsDict setObject: dict forKey: @"untitled"];
  [[mapWindow mapView] addWayPoint: dict];
}

- (void)deleteWayPoint:(id)sender
{
  WayPoint *wpoint = [[mapWindow mapView] selectedWayPoint];

  if (wpoint) {
    [wayPointsDict removeObjectForKey: [wpoint name]];
    [[mapWindow mapView] removeWayPoint: wpoint];
    [self changedSelectedWayPoint: nil];
  }
}

- (void)updateDictionaryForWayPoint:(WayPoint *)wpoint
                            oldName:(NSString *)oldname
{
  NSMutableDictionary *dict = [NSMutableDictionary dictionary];
  NSString *wname = [wpoint name];
  
  [dict setObject: wname forKey: @"name"];
  [dict setObject: [wpoint description] forKey: @"description"];
  [dict setObject: [wpoint iconName] forKey: @"iconname"];
  [dict setObject: [NSNumber numberWithFloat: [wpoint longitude]] 
           forKey: @"longitude"];
  [dict setObject: [NSNumber numberWithFloat: [wpoint latitude]] 
           forKey: @"latitude"];
  [dict setObject: [NSNumber numberWithLong: [wpoint scale]] 
           forKey: @"scale"];
  
  if (oldname) {
    [wayPointsDict removeObjectForKey: oldname];
  }

  [wayPointsDict setObject: dict forKey: wname];
  
  [self changedSelectedWayPoint: wpoint];
}

- (void)changedSelectedWayPoint:(WayPoint *)wpoint
{
  [[NSNotificationCenter defaultCenter] 
          postNotificationName: GPSSelectedWayPointDidChange 
                        object: wpoint];
}

- (WayPoint *)selectedWayPoint
{
  return [[mapWindow mapView] selectedWayPoint];
}

- (WayPoint *)wayPointWithName:(NSString *)wname
{
  return [wayPointsDict objectForKey: wname];
}

- (NSArray *)wayPointsForCenterPoint:(NSPoint)cp atScale:(long)scale
{
  NSArray *names = [wayPointsDict allKeys];
  NSMutableArray *scaledp = [NSMutableArray array];
  NSMutableArray *points = [NSMutableArray array];
  int i;

  for (i = 0; i < [names count]; i++) {
    NSDictionary *pdict = [wayPointsDict objectForKey: [names objectAtIndex: i]];
    long scl = [[pdict objectForKey: @"scale"] longValue];

    if (scl >= scale) {
      [scaledp addObject: pdict];
    }
  }
  
  for (i = 0; i < [scaledp count]; i++) {
    NSDictionary *pdict = [scaledp objectAtIndex: i];
    float lon = [[pdict objectForKey: @"longitude"] floatValue];
    float lat = [[pdict objectForKey: @"latitude"] floatValue];
  
    if (earthPointFits(NSMakePoint(lon, lat), cp, scale)) {
      [points addObject: pdict];
    }
  }
  
  return [points count] ? points : nil;
}

- (NSArray *)wayPointIconsNames
{
  return [NSArray arrayWithObject: defIconName];
}

@end










