/* 
 * main.m
 * 
 * Copyright (c) 2003 Enrico Sersale <enrico@imago.ro>
 *
 * Author: Enrico Sersale <enrico@imago.ro>
 * Date: August 2003
 *
 * This file is part of the GNUstep Position application
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <Foundation/Foundation.h>
#include <AppKit/AppKit.h>
#include "Functions.h"
#include "Position.h"
#include "GNUstep.h"

#ifdef GNUSTEP 
  void createMenu();
#endif

float *earthRadius;

int main(int argc, char **argv, char **env)
{
  int i;
  
	CREATE_AUTORELEASE_POOL (pool);
  
  earthRadius = NSZoneMalloc(NSDefaultMallocZone(), sizeof(float) * 201);		

  for (i = -100; i <= 100; i++) {
    earthRadius[i + 100] = radiusForLatitude(i);
  }

  Position *position = [Position position];
	NSApplication *app = [NSApplication sharedApplication];
  
#ifdef GNUSTEP
	createMenu();
#else
  [NSBundle loadNibNamed: @"MainMenu"  owner: position];
#endif
	
  [app setDelegate: position];    
	[app run];
  
  NSZoneFree(NSDefaultMallocZone(), earthRadius);
  
	RELEASE (pool);
  
  return 0;
}

#ifdef GNUSTEP 
void createMenu()
{
  NSMenu *mainMenu;
	NSMenu *info, *edit, *view, *tools, *insectors, *routes;
	NSMenu *windows, *services;  
	NSMenuItem *menuItem;

	// Main
  mainMenu = AUTORELEASE ([[NSMenu alloc] initWithTitle: @"Position"]);
    	
	// Info 	
	menuItem = addItemToMenu(mainMenu, @"Info", @"", nil, @"");
	info = AUTORELEASE ([NSMenu new]);
	[mainMenu setSubmenu: info forItem: menuItem];	
	addItemToMenu(info, @"Info Panel...", @"", @"showInfo:", @"");
	addItemToMenu(info, @"Preferences...", @"", @"showPreferences:", @"");
	addItemToMenu(info, @"Help...", @"", nil, @"?");
	 
	// Edit
	menuItem = addItemToMenu(mainMenu, @"Edit", @"", nil, @"");
	edit = AUTORELEASE ([NSMenu new]);
	[mainMenu setSubmenu: edit forItem: menuItem];	
	addItemToMenu(edit, @"Cut", @"", nil, @"x");
	addItemToMenu(edit, @"Copy", @"", nil, @"c");
	addItemToMenu(edit, @"Paste", @"", nil, @"v");

	// View
	menuItem = addItemToMenu(mainMenu, @"View", @"", nil, @"");
	view = AUTORELEASE ([NSMenu new]);
	[mainMenu setSubmenu: view forItem: menuItem];	
	addItemToMenu(view, @"Zoom in", @"", @"zoomIn:", @"z");
	addItemToMenu(view, @"Zoom out", @"", @"zoomOut:", @"Z");
				
	// Tools
	menuItem = addItemToMenu(mainMenu, @"Tools", @"", nil, @"");
	tools = AUTORELEASE ([NSMenu new]);
	[mainMenu setSubmenu: tools forItem: menuItem];	

  addItemToMenu(tools, @"Connect to gpsd", @"", @"connectToGpsdHost:", @"");
  addItemToMenu(tools, @"Disconnect from gpsd", @"", @"disconnectFromGpsdHost:", @"");  
  menuItem = addItemToMenu(tools, @"Inspectors", @"", nil, @"");
  insectors = AUTORELEASE ([NSMenu new]);
  [tools setSubmenu: insectors forItem: menuItem];	  
  addItemToMenu(insectors, @"Show Inspectors", @"", @"showInspectors:", @"i");
  addItemToMenu(insectors, @"Satellites", @"", @"showSatInspector:", @"1");
  addItemToMenu(insectors, @"Navigation", @"", @"showNavigationInspector:", @"2");
  addItemToMenu(insectors, @"Waypoints", @"", @"showWPointInspector:", @"3");
  addItemToMenu(insectors, @"Routes", @"", @"showRouteInspector:", @"4");
	addItemToMenu(tools, @"Download map", @"", @"showMapDownload:", @"");
	addItemToMenu(tools, @"Import map", @"", @"showMapImport:", @"");
  menuItem = addItemToMenu(tools, @"Routes", @"", nil, @"");
  routes = AUTORELEASE ([NSMenu new]);
  [tools setSubmenu: routes forItem: menuItem];	  
	addItemToMenu(routes, @"Add way point", @"", @"createWayPoint:", @"");
	addItemToMenu(routes, @"Remove way point", @"", @"deleteWayPoint:", @"");
	addItemToMenu(routes, @"Next point in route", @"", @"nextPointInRoute:", @"n");

	// Windows
	menuItem = addItemToMenu(mainMenu, @"Windows", @"", nil, @"");
	windows = AUTORELEASE ([NSMenu new]);
	[mainMenu setSubmenu: windows forItem: menuItem];		
	addItemToMenu(windows, @"Arrange in Front", @"", nil, @"");
	addItemToMenu(windows, @"Miniaturize Window", @"", nil, @"");
	addItemToMenu(windows, @"Close Window", @"", @"closeMainWin:", @"w");

	// Services 
	menuItem = addItemToMenu(mainMenu, @"Services", @"", nil, @"");
	services = AUTORELEASE ([NSMenu new]);
	[mainMenu setSubmenu: services forItem: menuItem];		

	// Hide
	addItemToMenu(mainMenu, @"Hide", @"", @"hide:", @"h");
	
	// Quit
	addItemToMenu(mainMenu, @"Quit", @"", @"terminate:", @"q");

	[mainMenu update];

	[[NSApplication sharedApplication] setServicesMenu: services];
	[[NSApplication sharedApplication] setWindowsMenu: windows];
	[[NSApplication sharedApplication] setMainMenu: mainMenu];		
}

#endif

